angular.module('lms.controllers').controller('ChargepointCtrl', ['$scope', '$http', '$stateParams', 'convert', '$state', '$filter', '$ionicActionSheet', 'general', '$ionicHistory', 'gps', 'appData', 
																function($scope, $http, $stateParams, convert, $state, $filter, $ionicActionSheet, general, $ionicHistory, gps, appData) {
	// Declaration scope variables
	$scope.guest = window.localStorage['guest'];
	$scope.chargepoints = [];
	$scope.chargepointIds = [$stateParams.chargePointId];
	$scope.loading = 'loading';
	$scope.transactionId = window.localStorage['transactionId'];
	$scope.showStartButton = false;
	$scope.errorMessage = "";
	
	// Scope functions
	$scope.startNavigation = function() {
		longitude = $scope.chargepoints[0].longitude;
		latitude = $scope.chargepoints[0].latitude;
		if (ionic.Platform.isAndroid()) { //  android
			navigator.google_navigate.navigate(latitude+","+longitude, function() {
			}, function(errMsg) {
				alert("error google maps");
			});
		}
		else if (ionic.Platform.isIOS()){ // ios
			window.open('maps:daddr='+latitude+","+longitude+'', '_system');
		}
		else {
			window.open('http://maps.apple.com/?daddr='+latitude+","+longitude+'&saddr='+gps.getLatitude()+','+gps.getLongitude(), '_system');
		}
	}
	
	$scope.loadContent = function() {
	  $http({
		url: general.server(),
		method: "POST",
		data: {action:'getChargePoints',parameters:{token:window.localStorage['token'], version: appData.appVersion, deviceId: appData.uuid, refLatitude: gps.getLatitude(), refLongitude: gps.getLongitude(), searchChargePointIds: $scope.chargepointIds, startResult: 0, endResult: 1, sort: "distance", availability: "all"}},
		headers: {'Content-Type': 'application/json'}
	  }).success(function (data, status, headers, config) {
			if (status == 200 || status == 304) {
				if (data['status'] == "valid") {
					if (data['chargePoints'].length == 0) {
						general.alert($filter('translate')('chargepoint.chargepointDeletedTitle'), $filter('translate')('chargepoint.chargepointDeletedInfo'));
						$ionicHistory.goBack();
						return;
					}					
					$scope.loading = 'loaded';
					$scope.chargepoints = [];
					angular.forEach(data['chargePoints'],function(chargepoint){
							 $scope.chargepoints.push({
								chargePointAddress: chargepoint.address,
								chargePointZipCode: chargepoint.zipCode,
								chargePointCity: chargepoint.city,
								chargePointId: chargepoint.chargePointId,
								chargePointVisibleId: chargepoint.chargePointVisibleId,
								latitude: chargepoint.latitude,
								longitude: chargepoint.longitude,
								provider: chargepoint.provider,
								url: chargepoint.url,
								openingHours: convert.openStatus(chargepoint.openingHours),
								channelsAvailable: chargepoint.channelsAvailable,
								available: convert.chargepointAvailable(chargepoint.channelsUnknown, chargepoint.channelsAvailable,chargepoint.channelsOccupied, chargepoint.channelsFailed),
								occupied: convert.chargepointOccupied(chargepoint.channelsUnknown, chargepoint.channelsAvailable,chargepoint.channelsOccupied, chargepoint.channelsFailed),
								unknown: convert.chargepointUnknown(chargepoint.channelsUnknown, chargepoint.channelsAvailable,chargepoint.channelsOccupied, chargepoint.channelsFailed),
								failed: convert.chargepointFailed(chargepoint.channelsUnknown, chargepoint.channelsAvailable,chargepoint.channelsOccupied, chargepoint.channelsFailed),
								connector: chargepoint.connector,
								distance: convert.chargepointDistance(chargepoint.distance, false),
								power: chargepoint.power,
								startTariff: convert.valuta(chargepoint.startTariff, chargepoint.vat),
								kWhTariff: convert.valuta(chargepoint.kWhTariff, chargepoint.vat),
								minuteTariff: convert.valuta(chargepoint.minuteTariff, chargepoint.vat),
								connectionTariff: convert.valuta(chargepoint.connectionTariff, chargepoint.vat),
								connectionTariffPeriod: chargepoint.connectionTariffPeriod,
								prepaidStartTariff: convert.valuta(chargepoint.prepaidStartTariff, chargepoint.vat),
								prepaidKWhTariff: convert.valuta(chargepoint.prepaidKWhTariff, chargepoint.vat),
								paymentMethods: chargepoint.paymentMethods
							});
					});
					$scope.alowedToStartTransaction();
				}
				else if (data['status'] == "versionTooLow") {
					general.versionToLow(); 
				}
				else {
					general.softlogout();
				}
			}
			 $scope.$broadcast('scroll.refreshComplete');
		}).error(function (data, status, headers, config) {
			$scope.loading = 'error';
			//alert(status);
			//alert("fout");
			$scope.$broadcast('scroll.refreshComplete');
		});
	}
	
	$scope.alowedToStartTransaction = function () {
		if ($scope.chargepoints[0].paymentMethods.length < 1) {
			$scope.showStartButton = false;
			$scope.errorMessage = $filter('translate')('chargepoint.noRemoteTransactions');
		}
		else if ($scope.chargepoints[0].available != true) {
			$scope.showStartButton = false;
			$scope.errorMessage = $filter('translate')('chargepoint.noChargepointsAvailable');
		}	
		else {
			$scope.showStartButton = true;
		}		
	}
	
	$scope.startTransaction = function () {
		if ($scope.showStartButton != true) {
			general.alert($filter('translate')('login.errortitle'), $scope.errorMessage);
		}
		else {
			$scope.selectPaymentMethod();
		}
	}
	
	$scope.selectPaymentMethod = function () {
		array = [];
		if ($scope.chargepoints[0].paymentMethods.indexOf("free") != -1) {
			if ($scope.chargepoints[0].paymentMethods.length == 1) {
				$state.go('app.transactionsettingsfree', {"chargePointId": $scope.chargepointIds, "channelsAvailable": $scope.chargepoints[0].channelsAvailable, "startTariff": null, "kWhTariff": null, "minuteTariff": null});
			}
			else {
				array.push({text : $filter('translate')('chargepoint.free')});
			}
		}
		if ($scope.chargepoints[0].paymentMethods.indexOf("invoice") != -1) {
			if ($scope.chargepoints[0].paymentMethods.length == 1) {
				$state.go('app.transactionsettingsinvoice', {"chargePointId": $scope.chargepointIds, "channelsAvailable": $scope.chargepoints[0].channelsAvailable, "startTariff": $scope.chargepoints[0].startTariff, "kWhTariff": $scope.chargepoints[0].kWhTariff, "minuteTariff": $scope.chargepoints[0].minuteTariff});
			}
			else {
				array.push({text : $filter('translate')('chargepoint.invoice')});
			}
		} 
		if ($scope.chargepoints[0].paymentMethods.indexOf("paypal") != -1) {
			if ($scope.chargepoints[0].paymentMethods.length == 1) {
				$state.go('app.transactionsettingspaypal', {"chargePointId": $scope.chargepointIds, "channelsAvailable": $scope.chargepoints[0].channelsAvailable, "startTariff": $scope.chargepoints[0].prepaidStartTariff, "kWhTariff": $scope.chargepoints[0].prepaidKWhTariff, "minuteTariff": null});
			}
			else {
				array.push({text : $filter('translate')('chargepoint.paypal')});
			}
		}
		$scope.showactionsheet(array);
	}
	
	$scope.showactionsheet = function(array) {
	   // Show the action sheet
	   var hideSheet = $ionicActionSheet.show({
		 buttons: array,
		 titleText: $filter('translate')('chargepoint.selectpaymentmethod'),
		 cancelText: $filter('translate')('chargepoint.cancel'),
		 cancel: function() {
			  // add cancel code..
			},
		 buttonClicked: function(index) {
			if (array[index].text == $filter('translate')('chargepoint.free')) {
				$state.go('app.transactionsettingsfree', {"chargePointId": $scope.chargepointIds, "channelsAvailable": $scope.chargepoints[0].channelsAvailable, "startTariff": null, "kWhTariff": null, "minuteTariff": null});
			}
			else if (array[index].text == $filter('translate')('chargepoint.invoice')) {
				$state.go('app.transactionsettingsinvoice', {"chargePointId": $scope.chargepointIds, "channelsAvailable": $scope.chargepoints[0].channelsAvailable, "startTariff": $scope.chargepoints[0].startTariff, "kWhTariff": $scope.chargepoints[0].kWhTariff, "minuteTariff": $scope.chargepoints[0].minuteTariff});
			}
			else if (array[index].text == $filter('translate')('chargepoint.paypal')) {
				$state.go('app.transactionsettingspaypal', {"chargePointId": $scope.chargepointIds, "channelsAvailable": $scope.chargepoints[0].channelsAvailable, "startTariff": $scope.chargepoints[0].prepaidStartTariff, "kWhTariff": $scope.chargepoints[0].prepaidKWhTariff, "minuteTariff": null});
			}
		 }
	   });
	 }
	 
	 // Init function
	 $scope.init = function () {
		gps.determineLocation(false, false);
		$scope.loadContent();
		general.keepScreenOn(false);
	 }
}]);