angular.module('lms.controllers').controller('MapCtrl', ['$scope', '$state', '$http', 'convert', '$ionicPopup', '$timeout', '$filter', 'general', 'gps', 'appData', '$ionicSideMenuDelegate', '$ionicLoading',
														function($scope, $state, $http, convert, $ionicPopup, $timeout, $filter, general, gps, appData, $ionicSideMenuDelegate, $ionicLoading) {
	
	// Declaration of scope variables
	$scope.map = null;
	$scope.mapLoaded = false;
	$scope.markerClusterer = null;
	$scope.mapRadius = 0;
	$scope.mapRadiusFactor = 2;
	$scope.newMapRadius = 0;
	$scope.chargepoints = [];
	$scope.centerChanged = false;
	$scope.follow = true;
	$scope.location = undefined;
	$scope.radius = undefined;
	$scope.loadContentVar = false;
	$scope.getChargepointsLatitude  = 0;
	$scope.getChargepointsLongitude = 0;
	$scope.lastLatitude = 0;
	$scope.lastLongitude = 0;
	$scope.moving = false;
	$scope.lock = false;
	$scope.updateMapButton = false;
	$scope.stopAutoUpdate = false;
	
	$scope.styles = [{
        url: appData.style + 'cluster.png',
        height: 60,
        width: 60,
        anchor: [0, 0],
        textColor: '#FFFFFF',
        textSize: 12
    }];
	
	var image_available = {
		url: 'img/icon/e-bike-charging-available.png',
		// This marker is 20 pixels wide by 32 pixels high.
		size: new google.maps.Size(32, 37),
		// The origin for this image is (0, 0).
		origin: new google.maps.Point(0, 0),
		// The anchor for this image is the base of the flagpole at (0, 32).
		anchor: new google.maps.Point(16, 37)
	};
	var image_unknown = {
		url: 'img/icon/e-bike-charging-unknown.png',
		// This marker is 20 pixels wide by 32 pixels high.
		size: new google.maps.Size(32, 37),
		// The origin for this image is (0, 0).
		origin: new google.maps.Point(0, 0),
		// The anchor for this image is the base of the flagpole at (0, 32).
		anchor: new google.maps.Point(16, 37)
	};
	var image_occupied = {
		url: 'img/icon/e-bike-charging-occupied.png',
		// This marker is 20 pixels wide by 32 pixels high.
		size: new google.maps.Size(32, 37),
		// The origin for this image is (0, 0).
		origin: new google.maps.Point(0, 0),
		// The anchor for this image is the base of the flagpole at (0, 32).
		anchor: new google.maps.Point(16, 37)
	};
	var image_failed = {
		url: 'img/icon/e-bike-charging-failed.png',
		// This marker is 20 pixels wide by 32 pixels high.
		size: new google.maps.Size(32, 37),
		// The origin for this image is (0, 0).
		origin: new google.maps.Point(0, 0),
		// The anchor for this image is the base of the flagpole at (0, 32).
		anchor: new google.maps.Point(16, 16)
	};
	var mylocation = {
		url: 'img/icon/location.png',
		// This marker is 20 pixels wide by 32 pixels high.
		size: new google.maps.Size(19, 28),
		// The origin for this image is (0, 0).
		origin: new google.maps.Point(0, 0),
		// The anchor for this image is the base of the flagpole at (0, 32).
		anchor: new google.maps.Point(9, 14)
	};
	var shape = {
		coords: [1, 1, 1, 20, 18, 20, 18, 1],
		type: 'poly'
	};
	
	var positionMovingIcon = {
		path: 'M1502 6097 l-932 -932 88 -88 88 -88 115 82 c350 252 548 333 1107 455 323 70 611 70 934 0 552 -120 754 -202 1088 -441 63 -45 120 -84 126 -88 6 -3 49 33 97 81 l87 87 -933 933 -932 932 -933 -933z M2190 4895 c-383 -48 -756 -205 -1052 -443 -481 -388 -767 -988 -767 -1607 0 -544 219 -1073 605 -1459 386 -386 915 -605 1459 -605 544 0 1073 219 1459 605 170 170 289 337 396 559 280 575 280 1225 0 1800 -208 428 -527 747 -955 955 -359 174 -759 243 -1145 195z',
		fillColor: '#4285f4',
		fillOpacity: 0.95,
		scale: 0.0040,
		strokeColor: 'white',
		strokeWeight: 1,
		strokeOpacity: 0.75,
		anchor: new google.maps.Point(2500, 2500)
	};
	
	// Generic functions:
	var rad = function(x) {
		return x * Math.PI / 180;
	};

	var getDistance = function(p1, p2) {
	  var R = 6378137; // Earth’s mean radius in meter
	  var dLat = rad(p2.lat() - p1.lat());
	  var dLong = rad(p2.lng() - p1.lng());
	  var a = Math.sin(dLat / 2) * Math.sin(dLat / 2) +
		Math.cos(rad(p1.lat())) * Math.cos(rad(p2.lat())) *
		Math.sin(dLong / 2) * Math.sin(dLong / 2);
	  var c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1 - a));
	  var d = R * c;
	  return d; // returns the distance in meter
	};
	
	var convertRadius = function (radius) {
		return (parseInt(radius * Math.max(0.5, ($scope.map.getZoom()*0.5) - 4.5), 10));
	}
	
	var filterMax = function(arr, max) {
		return arr.filter(function(item) {
			return item < max;
		});
	}

	var zoomTimer = null;
	var dragendTimer = null;
	// Scope functions
	$scope.loadMap = function() {
		$scope.map = new google.maps.Map(document.getElementById('map-canvas'), {
			zoom: 15,
			center: new google.maps.LatLng(gps.getLatitude(), gps.getLongitude()),
			disableDefaultUI: true,
			rotateControl: true
		});
		
		$scope.map.addListener('dragstart', function() {
			clearTimeout(dragendTimer);
			$scope.moving = true;
			$scope.centerChanged = true;
			$scope.follow = false;
			$scope.$apply();
		});
		  
		$scope.map.addListener('zoom_changed', function() {
				clearTimeout(zoomTimer);
				zoomTimer = setTimeout(function() {
					$scope.newMapRadius = convertRadius(getDistance($scope.map.getCenter(), new google.maps.LatLng($scope.map.getBounds().f.b, $scope.map.getBounds().b.b)));
					if ($scope.newMapRadius > 15000) {
						$scope.newMapRadius = 15000;
					}
					if ($scope.newMapRadius > ($scope.mapRadius)) {
						console.log("Oke we moeten opnieuw gaan laden!");
						$scope.loadContent($scope.map.getCenter().lat(), $scope.map.getCenter().lng());
					}
					else {
						if ($scope.chargepoints.length > 1000) {
							console.log("We should do some cleaning!!!");
							$scope.deletePinsOutOfRange($scope.newMapRadius);
							$scope.mapRadius = $scope.newMapRadius
						}
					}
					console.log("Map radius:" + getDistance($scope.map.getCenter(), new google.maps.LatLng($scope.map.getBounds().f.b, $scope.map.getBounds().b.b)) + " getzoom:" + $scope.map.getZoom());
				}, 1000);
				
		});
		  
		$scope.map.addListener('dragend', function() {
				dragendTimer = setTimeout(function() {
					var totalMovement = getDistance($scope.map.getCenter(), new google.maps.LatLng($scope.getChargepointsLatitude, $scope.getChargepointsLongitude));
					if (totalMovement > (($scope.mapRadiusFactor - 1) * parseInt($scope.mapRadius, 10))) {
						$scope.loadContent($scope.map.getCenter().lat(), $scope.map.getCenter().lng());
					}
				}, 1000);
		});
		
		google.maps.event.addListenerOnce($scope.map, 'idle', function(){
			// do something only the first time the map is loaded
			$scope.mapLoaded = true;
			console.log("Load map");
			$scope.setLocation();
			$scope.newMapRadius = convertRadius(getDistance($scope.map.getCenter(), new google.maps.LatLng($scope.map.getBounds().f.b, $scope.map.getBounds().b.b)));
			if (!gps.isActual()) {
				$ionicLoading.show({
				  template: 'Bezig met het bepalen van uw locatie...',
				  duration: 5000
				});
				$scope.loadContentVar = true; // we didn't load the content during init, so do it during a new location. see event
				$scope.map.setZoom(8);
			}
			else {
				$scope.loadContent(gps.getLatitude(), gps.getLongitude());
			}
		});
	};
	
	$scope.loadContent = function(latitude, longitude) {
	  $scope.updateMapButton = false;
	  $scope.getChargepointsLatitude  = latitude;
	  $scope.getChargepointsLongitude = longitude;
	  
	  returnFields = ["chargePointId", "latitude", "longitude", "channelsAvailable", "channelsOccupied", "channelsUnknown", "channelsFailed", "distanceFromSearch"];
	  
	  $scope.mapRadius = $scope.newMapRadius;
	  
	  $http({
		url: general.server(),
		method: "POST",
		data: {action:'getChargePoints',parameters:{token:window.localStorage['token'], version: appData.appVersion, deviceId: appData.uuid, refLatitude: gps.getLatitude() , refLongitude: gps.getLongitude(), searchLatitude: $scope.getChargepointsLatitude, searchLongitude: $scope.getChargepointsLongitude, radius: $scope.mapRadius, startResult: 0, endResult: 1000, sort: "distance", availability: convert.availability(window.localStorage['searchavailable'] == 'true'), connectors: JSON.parse(window.localStorage['searchconnectorArray']), minPower: window.localStorage['searchchargeEnergy'] * 1000, remoteManageable: (window.localStorage['searchpaywithapp'] == 'true'), returnFields: returnFields}},
		headers: {'Content-Type': 'application/json'}
	  }).success(function (data, status, headers, config) {
			if (status == 200 || status == 304) {
				if (data['status'] == "valid") {
					$scope.loading = false;
					$scope.chargepoints = [];
					angular.forEach(data['chargePoints'],function(chargepoint){
						$scope.chargepoints.push({
							chargePointId: chargepoint.chargePointId,
							available: convert.chargepointAvailable(chargepoint.channelsUnknown, chargepoint.channelsAvailable,chargepoint.channelsOccupied, chargepoint.channelsFailed),
							occupied: convert.chargepointOccupied(chargepoint.channelsUnknown, chargepoint.channelsAvailable,chargepoint.channelsOccupied, chargepoint.channelsFailed),
							unknown: convert.chargepointUnknown(chargepoint.channelsUnknown, chargepoint.channelsAvailable,chargepoint.channelsOccupied, chargepoint.channelsFailed),
							failed: convert.chargepointFailed(chargepoint.channelsUnknown, chargepoint.channelsAvailable,chargepoint.channelsOccupied, chargepoint.channelsFailed),
							latitude: chargepoint.latitude,
							longitude: chargepoint.longitude,
							distanceFromSearch: chargepoint.distanceFromSearch
						});
					});
					if ($scope.chargepoints.length == 0) {
						return;
					}
					console.log("Place pins");
					$scope.placePins();
				}
				else if (data['status'] == "versionTooLow") {
					general.versionToLow(); 
				}
				else {
					general.softlogout();
				}
				$scope.updateMapButton = true;
			}
		}).error(function (data, status, headers, config) {
			//alert(status);
			//alert("fout");
			$scope.updateMapButton = true;
		});
	};
	
	var markers = [];
	
	$scope.placePins = function() {
		$scope.deletePins();
		if ($scope.markerClusterer) {
			$scope.markerClusterer.clearMarkers();
        }
		angular.forEach($scope.chargepoints,function(chargepoint){
			if (chargepoint.available) {
				var marker = new google.maps.Marker({
					map: $scope.map,
					icon: image_available,
					position: {lat: chargepoint.latitude, lng: chargepoint.longitude}
				});
			}
			else if (chargepoint.unknown) {
				var marker = new google.maps.Marker({
					map: $scope.map,
					icon: image_unknown,
					position: {lat: chargepoint.latitude, lng: chargepoint.longitude}
				});
			}
			else {
				var marker = new google.maps.Marker({
					map: $scope.map,
					icon: image_occupied,
					position: {lat: chargepoint.latitude, lng: chargepoint.longitude}
				});
			}
			marker.addListener('mousedown', function() {
				$scope.showChargepointInfo(chargepoint.chargePointId, chargepoint.address, chargepoint.city, chargepoint.available, chargepoint.occupied, chargepoint.unknown, chargepoint.power, chargepoint.connector, chargepoint.failed, chargepoint.chargePointVisibleId);
			});
			markers.push(marker);
		});
		$scope.markerClusterer = new MarkerClusterer($scope.map, markers, {
			  maxZoom: 14,
			  gridSize: 50,
			  styles: $scope.styles
		});
	};
	
	$scope.deletePins = function() {
		if ($scope.markerClusterer != null) {
			$scope.markerClusterer.clearMarkers();
			markers = [];
		}
	};
	
	$scope.deletePinsOutOfRange = function(range) {
		var index = 0;
		var pinsRemoved = 0;
		angular.forEach($scope.chargepoints,function(chargepoint) {
			if (chargepoint.distanceFromSearch > range) {
				delete $scope.chargepoints[index];
				pinsRemoved++;
			}
			index++;
		});
		console.log("Removed " + pinsRemoved + "/" + index);
		$scope.placePins();
	};
	
  
    $scope.showChargepointInfo = function(chargePointId, address, city, available, occupied, unknown, power, connector, failed, chargePointVisibleId) {
		/*
	  var confirmPopup = $ionicPopup.confirm({
		 title: $filter('translate')('map.title'),
		 template: ' <i class="icon icon-small ion-location"></i> ' + chargePointVisibleId + '</br><i class="icon icon-small ion-android-locate"></i> ' + address + ', ' + city + '</br><i class="icon icon-small ion-ios-bolt"></i>  Max. ' + convert.energy(power) + ' </br> <i class="icon icon-small fa-plug"></i> ' + convert.connectorString(connector) + ' <p ng-show="'+available+'==true"><i class="icon icon-small ion-checkmark-circled"></i>' + $filter('translate')('map.statusavailable')  + '</p><p ng-show="'+occupied+'==true"><i class="icon icon-small ion-close-circled"></i>' + $filter('translate')('map.statusoccupied')  + '</p><p ng-show="'+unknown+'==true"><i class="icon icon-small ion-help-circled"></i>' + $filter('translate')('map.statusunknown')  + '</p>' + '<p ng-show="'+failed+'==true"><i class="icon icon-small ion-help-circled"></i>' + $filter('translate')('map.statusfailed')  + '</p>',
		 cancelText: $filter('translate')('map.cancel'), // String (default: 'Cancel'). The text of the Cancel button.
		 cancelType: 'button-default', // String (default: 'button-default'). The type of the Cancel button.
		 okText: $filter('translate')('map.proceed'), // String (default: 'OK'). The text of the OK button.
		 okType: 'button-positive' // String (default: 'button-positive'). The type of the OK button.
	   });
	   confirmPopup.then(function(res) {
		 if(res) {
		   $state.go('app.chargepoint', { "chargePointId": chargePointId});
		 } else {
		   // do nothing
		 }
	   });
	   */
	    $state.go('app.chargepoint', { "chargePointId": chargePointId});
	 };
	
	$scope.zoomIn = function () {
		$scope.map.setZoom($scope.map.getZoom() + 1);
	 };
	 
	$scope.zoomOut = function() {
		$scope.map.setZoom($scope.map.getZoom() - 1);
	 };
	
	$scope.centerPosition = function() {
		$scope.map.panTo(new google.maps.LatLng(gps.getLatitude(), gps.getLongitude()));
	 };
	
	$scope.followPosition = function () {
		if ($scope.follow == false) {
			$scope.follow = true;
		}
		else {
			$scope.follow = false;
		}
	 };
	 
	$scope.setLocation = function() {
		if (gps.isActual() && $scope.mapLoaded) {
			if ($scope.location == undefined) {
				console.log("Set my location on the map");
				
				$scope.location = new google.maps.Marker({
						map: $scope.map,
						icon: positionMovingIcon,
						position: {lat: gps.getLatitude(), lng: gps.getLongitude()},
						rotation: -180
				});
	
				$scope.location.setPosition(new google.maps.LatLng(gps.getLatitude(), gps.getLongitude()));
				
				$scope.radius = new google.maps.Circle({
					  strokeColor: '#2693e7',
					  strokeOpacity: 0.6,
					  strokeWeight: 2,
					  fillColor: '#2693e7',
					  fillOpacity: 0.2,
					  map: $scope.map,
					  center: new google.maps.LatLng(gps.getLatitude(), gps.getLongitude()),
					  radius: Number(gps.getAccuracy())
					});
			}
			else {
				console.log("Move my location on the map");
				
				if (!(isNaN(gps.getHeading()))) {
					positionMovingIcon.rotation = gps.getHeading() - 180;
				}
				$scope.location.setIcon(positionMovingIcon);
				$scope.location.setPosition(new google.maps.LatLng(gps.getLatitude(), gps.getLongitude()));
				$scope.radius.setCenter(new google.maps.LatLng(gps.getLatitude(), gps.getLongitude()));
				$scope.radius.setRadius(Number(gps.getAccuracy()));	
								
			}
		}
	}
	
	$scope.updateMap = function () {
		$scope.stopAutoUpdate = true;
		console.log("stop auto update");
		$scope.loadContent($scope.map.getCenter().lat(), $scope.map.getCenter().lng());
	}
	
	$scope.clickButton = function () {
		$scope.stopAutoUpdate = false;
		console.log("start auto update");
		if ($scope.centerChanged != true) {
			$scope.followPosition();
		}
		else {
			$scope.centerPosition();
			$scope.centerChanged = false;
		}
		if ((Math.abs($scope.getChargepointsLatitude - gps.getLatitude()) > 0.01 || Math.abs($scope.getChargepointsLongitude - gps.getLongitude()) > 0.01)) {
			$scope.loadContent(gps.getLatitude(), gps.getLongitude(), $scope.map.getZoom());
		}
	 };
	
	 //event from GPS service

	 gps.newLocationCallback('newLocation', $scope, function() { // new GPS location
		console.log("new location");
		if ($scope.loadContentVar) {
			$scope.loadContentVar  = false;
			$scope.map.setZoom(13);
			$ionicLoading.hide();
			$scope.loadContent(gps.getLatitude(), gps.getLongitude());
		}
		
		$scope.setLocation(); // Set the new location of the location icon
		if ($scope.follow) {
			$scope.centerPosition();
		}
		
     });
	 
	 gps.newLocationCallback('newCompass', $scope, function() { // new cmpass heading
		console.log("new Compass");
		if ($scope.location != undefined) {
			positionMovingIcon.rotation = gps.getHeading() - 180;
			$scope.location.setIcon(positionMovingIcon);			
		}
	 });
	 
	 // Init function
	 
	 $scope.init = function() {
		console.log("map init");
		$scope.location = undefined;
		$scope.styles[0].url = appData.style + 'cluster.png';
		$ionicSideMenuDelegate.canDragContent(false);
		general.keepScreenOn(true);
		gps.startWatchLocation();
		$scope.loadMap();
		
	 }
	 
	$scope.$on('$ionicView.enter', function(){
		$scope.init();
	});
	 
	 // Deinit function
	 
	 $scope.deinit = function() {
		$ionicSideMenuDelegate.canDragContent(true);
		general.keepScreenOn(false);
	 }
	 
	 $scope.$on('$ionicView.leave', function(){
	    $scope.deinit();
	});

	
}]);