angular.module('lms.controllers').controller('SearchQueryCtrl', ['$scope', '$stateParams', '$http', 'convert', '$filter', 'general', 'gps', 'appData',
																function($scope, $stateParams, $http, convert, $filter, general, gps, appData) {
	// Declaration of scope variables
	$scope.input = $stateParams.input;
	$scope.startResult = $stateParams.startResult;
	$scope.endResult = $stateParams.endResult;
	$scope.results = window.localStorage['results'];
	$scope.chargepoints = [];
	$scope.searchHistory = [];
	$scope.loading = 'loading';
	$scope.moreResults = false;
	
	//Scope functions
	$scope.addToHistory = function(searchString) {
		if(((localStorage["searchHistory"]) != undefined) && (localStorage["searchHistory"]) != "") {
			$scope.searchHistory = JSON.parse(localStorage["searchHistory"]);
			if ($scope.searchHistory.indexOf(searchString) != -1) {
				// This search result is already in the history. Remove it from the history
				$scope.searchHistory.splice($scope.searchHistory.indexOf(searchString), 1);
			}
			if ($scope.searchHistory.length >= 10) {
				$scope.searchHistory.splice(10, 1);
			}
		}
		$scope.searchHistory.push(searchString);
		localStorage["searchHistory"] = JSON.stringify($scope.searchHistory);
	}
	
	$scope.loadContent = function(doRefresh) {
		if (doRefresh != true) {
			$scope.loading = 'loading';
		}
		if ($stateParams.input == "") {
			$scope.loadLocationContent();
		}
		else {
			$scope.loadSearchContent();
		}
	}
	
	$scope.loadLocationContent = function() {
	  
	  $http({
		url: general.server(),
		method: "POST",
		data: {action:'getChargePoints',parameters:{token:window.localStorage['token'], version: appData.appVersion, deviceId: appData.uuid, refLatitude: gps.getLatitude(), refLongitude: gps.getLongitude(), radius: 100000, searchLatitude: gps.getLatitude(), searchLongitude:gps.getLongitude() , startResult: $stateParams.startResult, endResult: $stateParams.endResult, sort: "distance", availability: convert.availability(window.localStorage['searchavailable'] == 'true'), connectors: JSON.parse(window.localStorage['searchconnectorArray']), minPower: window.localStorage['searchchargeEnergy'] * 1000, remoteManageable: (window.localStorage['searchpaywithapp'] == 'true')}},
		headers: {'Content-Type': 'application/json'}
	  }).success(function (data, status, headers, config) {
			if (status == 200 || status == 304) {
				$scope.loading = 'loaded';
				if (data['status'] == "valid") {
					if (data['chargePoints'].length != 0) {
						$scope.chargepoints = [];
						$scope.moreResults = data['moreResults'];
						angular.forEach(data['chargePoints'],function(chargePoint){
								 $scope.chargepoints.push({
									chargePointId: chargePoint.chargePointId,
									address: chargePoint.address,
									available: convert.chargepointAvailable(chargePoint.channelsUnknown, chargePoint.channelsAvailable,chargePoint.channelsOccupied, chargePoint.channelsFailed),
									occupied: convert.chargepointOccupied(chargePoint.channelsUnknown, chargePoint.channelsAvailable,chargePoint.channelsOccupied, chargePoint.channelsFailed),
									unknown: convert.chargepointUnknown(chargePoint.channelsUnknown, chargePoint.channelsAvailable,chargePoint.channelsOccupied, chargePoint.channelsFailed),
									failed: convert.chargepointFailed(chargePoint.channelsUnknown, chargePoint.channelsAvailable,chargePoint.channelsOccupied, chargePoint.channelsFailed),
									distance: convert.chargepointDistance(chargePoint.distance, true)
								});
						});
					}
					else {
						$scope.chargepoints = null;
						general.softlogout();
					}
					$scope.$broadcast('scroll.refreshComplete');
				}
			}
		}).error(function (data, status, headers, config) {
			$scope.loading = 'error';
			$scope.$broadcast('scroll.refreshComplete');
		});
	};
	
	$scope.loadSearchContent = function() {
	  
	  $http({
		url: general.server(),
		method: "POST",
		data: {action:'getChargePoints',parameters:{token:window.localStorage['token'], version: appData.appVersion, deviceId: appData.uuid, refLatitude: gps.getLatitude(), refLongitude: gps.getLongitude(), radius: 100000, searchString: $stateParams.input, startResult: $stateParams.startResult, endResult: $stateParams.endResult, sort: "distanceFromSearch", availability: convert.availability(window.localStorage['searchavailable'] == 'true'), connectors: JSON.parse(window.localStorage['searchconnectorArray']), minPower: window.localStorage['searchchargeEnergy'] * 1000, remoteManageable: (window.localStorage['searchpaywithapp'] == 'true')}},
		headers: {'Content-Type': 'application/json'}
	  }).success(function (data, status, headers, config) {
			if (status == 200 || status == 304) {
				$scope.loading = 'loaded';
				if (data['status'] == "valid") {
					if (data['chargePoints'].length != 0) {
						$scope.chargepoints = [];
						$scope.moreResults = data['moreResults'];
						angular.forEach(data['chargePoints'],function(chargePoint){
								 $scope.chargepoints.push({
									chargePointId: chargePoint.chargePointId,
									address: chargePoint.address,
									available: convert.chargepointAvailable(chargePoint.channelsUnknown, chargePoint.channelsAvailable,chargePoint.channelsOccupied, chargePoint.channelsFailed),
									occupied: convert.chargepointOccupied(chargePoint.channelsUnknown, chargePoint.channelsAvailable,chargePoint.channelsOccupied, chargePoint.channelsFailed),
									unknown: convert.chargepointUnknown(chargePoint.channelsUnknown, chargePoint.channelsAvailable,chargePoint.channelsOccupied, chargePoint.channelsFailed),
									failed: convert.chargepointFailed(chargePoint.channelsUnknown, chargePoint.channelsAvailable,chargePoint.channelsOccupied, chargePoint.channelsFailed),
									distance: convert.chargepointDistance(chargePoint.distance, true)
								});
						});
						$scope.addToHistory($stateParams.input);
					}
					else {
						$http({
							url: general.server(),
							method: "POST",
							data: {action:'getChargePoints',parameters:{token:window.localStorage['token'], version: appData.appVersion, deviceId: appData.uuid, refLatitude: gps.getLatitude(), refLongitude: gps.getLongitude(), radius: null, searchChargePointIds: [$stateParams.input], startResult: $stateParams.startResult, endResult: $stateParams.endResult, sort: "distance", availability: convert.availability(window.localStorage['searchavailable'] == 'true'), connectors: JSON.parse(window.localStorage['searchconnectorArray']), minPower: window.localStorage['searchchargeEnergy'] * 1000, remoteManageable: (window.localStorage['searchpaywithapp'] == 'true')}},
							headers: {'Content-Type': 'application/json'}
						  }).success(function (data, status, headers, config) {
								if (status == 200 || status == 304) {
									if (data['status'] == "valid") {
										if (data['chargePoints'].length != 0) {
											$scope.loading = 'loaded';
											$scope.moreResults = data['moreResults'];
											$scope.chargepoints = [];
											angular.forEach(data['chargePoints'],function(chargePoint){
													 $scope.chargepoints.push({
														chargePointId: chargePoint.chargePointId,
														address: chargePoint.address,
														available: convert.chargepointAvailable(chargePoint.channelsUnknown, chargePoint.channelsAvailable,chargePoint.channelsOccupied, chargePoint.channelsFailed),
														occupied: convert.chargepointOccupied(chargePoint.channelsUnknown, chargePoint.channelsAvailable,chargePoint.channelsOccupied, chargePoint.channelsFailed),
														unknown: convert.chargepointUnknown(chargePoint.channelsUnknown, chargePoint.channelsAvailable,chargePoint.channelsOccupied, chargePoint.channelsFailed),
														failed: convert.chargepointFailed(chargePoint.channelsUnknown, chargePoint.channelsAvailable,chargePoint.channelsOccupied, chargePoint.channelsFailed),
														distance: convert.chargepointDistance(chargePoint.distance, true)
													});
											});
											$scope.addToHistory($scope.searchquery);
										}
										else {
											$scope.chargepoints = null;
										}
									}
									else {
										general.softlogout();
									}
								}
							}).error(function (data, status, headers, config) {
								$scope.loading = 'error';
								$scope.$broadcast('scroll.refreshComplete');
							});
					}
					$scope.$broadcast('scroll.refreshComplete');
				}
			}
		}).error(function (data, status, headers, config) {
			$scope.loading = 'error';
			$scope.$broadcast('scroll.refreshComplete');
		});
	};
	
	$scope.doRefresh = function() {
		this.loadContent(true);
	};
	
	// Init function
	$scope.init = function() {
		gps.determineLocation(false, false);
	}
}]);