angular.module('gpsService', [], function($provide) {
    $provide.factory('gps', ['$rootScope', '$timeout', 'general', 'appData', function($rootScope, $timeout, general, appData) {
        var gps = {};
		var gpsErrorCounter = 0;
		var watchIDGps;
		var watchIDCompass;
		var watchActivated = false;
		var busy = false;
		var errorMessageShowed = false;
		
        gps.newLocationCallback = function(msg, scope, func) {
            var unbind = $rootScope.$on(msg, func);
            scope.$on('$destroy', unbind);
        };
		gps.getLongitude = function () {
			if (window.localStorage['longitude'] == undefined || window.localStorage['longitude'] == "" || window.localStorage['longitude'] == null) {
				return 4.87995;
			}
			else {
				return window.localStorage['longitude'];
			}
		};
		gps.getLatitude = function () {
			if (window.localStorage['latitude'] == undefined || window.localStorage['latitude'] == "" || window.localStorage['latitude'] == null) {
				return 52.37783;
			}
			else {
				return window.localStorage['latitude'];
			}
		};
		gps.getAccuracy = function () {
			if (window.localStorage['accuracy'] == undefined || window.localStorage['accuracy'] == "" || window.localStorage['accuracy'] == null) {
				return 1; 
			}
			else {
				return window.localStorage['accuracy'];
			}
		};
		gps.getHeading = function () {
			if (window.localStorage['heading'] == undefined || window.localStorage['heading'] == "" || window.localStorage['heading'] == null || isNaN(window.localStorage['heading']) || window.localStorage['heading'] > 360) {
				return 0; 
			}
			else {
				return window.localStorage['heading'];
			}
		};
		gps.getSpeed = function () {
			if (window.localStorage['speed'] == undefined || window.localStorage['speed'] == "" || window.localStorage['speed'] == null || isNaN(window.localStorage['speed'])) {
				return 0; 
			}
			else {
				return window.localStorage['speed'];
			}
		};
		gps.isActual = function () {
			var d = new Date();
			if ((d.getTime() - window.localStorage['time']) > 30000) {
				return false;
			}
			return true;
		};
		gps.badGpsSignal = function () {
			if (gpsErrorCounter >= 5) {
				return true;
			}
			else {
				return false;
			}
		};
		gps.onSuccessGps = function(position) {
			console.log("Success during gps");
			busy = false;
			window.localStorage['longitude'] 	= position.coords.longitude;
			window.localStorage['latitude']  	= position.coords.latitude;
			window.localStorage['accuracy']  	= position.coords.accuracy;
			if (gps.getSpeed() >= 1 && (isNaN(position.coords.heading) == false)) {
				window.localStorage['heading']  = position.coords.heading;
			}
			window.localStorage['speed']   		= position.coords.speed;
			window.localStorage['time']      	= position.timestamp;
			
			$rootScope.$emit("newLocation");
			if (gpsErrorCounter > 0) {
				gpsErrorCounter--;
			}
		};
		gps.onErrorGps = function(error) {
			console.log("Error during gps");
			busy = false;
			if ((error.code == 1 || gpsErrorCounter == 10) && !errorMessageShowed) {
				errorMessageShowed = true;
				general.toast("No GPS signal, turn on your GPS", 2000);
			}
			if (gpsErrorCounter <= 10) {
				gpsErrorCounter++;
			}
			if (!watchActivated) {
				gps.determineLocation(true, false);
			}
		};
		gps.onSuccessCompass = function(reading) {
			if ((gps.getSpeed() < 1) && (reading.magneticHeading >= 0)) {
				// We are moving very slow (< 1 m/s), so use the compass heading
				window.localStorage['heading']   = reading.magneticHeading;
				$rootScope.$emit("newCompass");
			}
		};
		gps.onErrorCompass = function(error) {
			//general.alert("No compass");
		};
		
		gps.determineLocation = function(forced, highAccuracy) {
			console.log("Determine new location");
			gps.stopWatchLocation();
			if (gps.isActual() == false || forced) {
				if (!busy) {
					busy = true;
					$timeout(function() {
						var options = { maximumAge: 1000, timeout: 2000, enableHighAccuracy: highAccuracy };
						navigator.geolocation.getCurrentPosition(gps.onSuccessGps, gps.onErrorGps, options);
					}, 100);
				}
			}
		};
		
		gps.startWatchLocation = function() {
			console.log("Start watch location");
			if (!watchActivated) {
				watchActivated = true;
				
				var optionsGps = { maximumAge: 1000, timeout: 1000, enableHighAccuracy: true };
				watchIDGps = navigator.geolocation.watchPosition(gps.onSuccessGps,gps.onErrorGps,optionsGps);
				
				if (appData.platform != "Windows") {
					var options = { frequency: 250 }; // Update every 250mss
					watchIDCompass = navigator.compass.watchHeading(gps.onSuccessCompass, gps.onErrorCompass, options);
				}
				
				console.log("watch activated");
			}
		};
		gps.stopWatchLocation = function() {
			if (watchActivated) {
				watchActivated = false;
				navigator.geolocation.clearWatch(watchIDGps);
				if (appData.platform != "Windows") {
					navigator.compass.clearWatch(watchIDCompass);
				}
				console.log("watch deactivated");
			}
		};
	
        return gps;
    }]);
})
