angular.module('intervalDataService', [], function($provide) {
    $provide.factory('intervalDataService', ['$rootScope', '$timeout', function($rootScope, $timeout) {
        var intervalDataService = {};
		
		// declaration variables
		var serviceCounter = 0;
		var timeout = [];
		var runningFlag = [];
		var standbyFlag = [];
		var timeoutTimer = [];
		var serviceName = [];
		var intervalFunction = [];
		
		//interrupt
		intervalDataService.newIntervalDataCallback = function(msg, scope, func) {
            var unbind = $rootScope.$on(msg, func);
            scope.$on('$destroy', unbind);
        };
		
		// Declare interval service
		intervalDataService.init = function(serviceNameVar, intervalFunctionVar) {
			serviceCounter++;
			timeout.push(5000);
			runningFlag.push(false);
			standbyFlag.push(false);
			timeoutTimer.push("NULL");
			serviceName.push(serviceNameVar);
			intervalFunction.push(intervalFunctionVar);
		}
		
		// start service
		intervalDataService.start = function (service, definedTimeout) {
			var index = serviceName.indexOf(service);
			if (index == -1) {
				return;
			}
			if (timeout[index] == definedTimeout && runningFlag[index] == true) {
				return;
			}
			console.log("start service:" + serviceName[index]);
			timeout[index] = definedTimeout;			// store defined timeout.
			if (runningFlag[index] != false) { 			// service is not running.
				console.log(" reset timer");
				$timeout.cancel(timeoutTimer[index]); 	// destroy current interval timer.
			}	
			runningFlag[index] = true;					// service is now running.
			intervalDataService.getData(index); 				// start to get data with the defined interval.
		};
		
		intervalDataService.stop = function (service) {
			var index = serviceName.indexOf(service);
			if (index == -1) {
				return;
			}
			if (runningFlag[index] == true) { 			// service is not running.
				console.log("stop service: " + serviceName[index]);
				runningFlag[index] = false;
				$timeout.cancel(timeoutTimer[index]); 	// destroy current interval timer.
			}	
		};
		
		intervalDataService.getData = function (index) {
			console.log("get interval data");
			
			intervalFunction[index](serviceName[index]);
			
			timeoutTimer[index] = $timeout(function() {
				intervalDataService.getData(index);
			}, timeout[index]);
		};
		
		intervalDataService.pause = function () {
			for (i = 0; i < serviceCounter; i++) {
				if (runningFlag[i] == true) {
					standbyFlag[i] = true;
					intervalDataService.stop(serviceName[i]);
				}
			}
		};
		
		intervalDataService.resume = function () {
			for (i = 0; i < serviceCounter; i++) {
				if (standbyFlag[i] == true) {
					standbyFlag[i] = false;
					intervalDataService.start(serviceName[i], timeout[i]);
				}
			}
		};
	
        return intervalDataService;
    }]);
})
